#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# keyfile_dialog.pyw
# 
# the user should give us a keyfile
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright 2013-2014 X-Net and AIT Austrian Institute of Technology
# 
# 
#     X-Net Services GmbH
#     Elisabethstrasse 1
#     4020 Linz
#     AUSTRIA
#     https://www.x-net.at
# 
#     AIT Austrian Institute of Technology
#     Donau City Strasse 1
#     1220 Wien
#     AUSTRIA
#     http://www.ait.ac.at
# 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#    http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import base64
import sys

if sys.platform == 'win32' or sys.platform == 'cygwin':
    import win32api
    import win32con
    import win32gui

from PyQt4 import QtCore
from PyQt4 import QtGui

from ui_KeyfileDialog import Ui_KeyfileDialog 
from about_dialog import AboutDialog


# ------------------------------------------------------------
# code


class KeyfileDialog(QtGui.QDialog):

    """A dialog for letting the user pass on a password/keyfile combo"""

    def __init__(self, user_text = 'Please provide an approbitate password and keyfile to proceed:'):

        QtGui.QDialog.__init__(self)

        # setup the user interface
        self.ui = Ui_KeyfileDialog()
        self.ui.setupUi(self)
    
        # local members
        self._about_dialog = AboutDialog()
        self._open_file_dialog = QtGui.QFileDialog(self, 'Open Keyfile', QtCore.QDir.homePath(), 'All Files (*.*);;')
        self._open_file_dialog.setAcceptMode(QtGui.QFileDialog.AcceptOpen)
        self._open_file_dialog.setFileMode(QtGui.QFileDialog.ExistingFile)

        # connectors
        self.ui.lblText.setText(user_text)
        self.ui.btnAbout.clicked.connect(self.clicked_about)
        self.ui.btnCancel.clicked.connect(self.reject)
        self.ui.btnOk.clicked.connect(self.clicked_ok)
        self.ui.btnBrowse.clicked.connect(self.clicked_browse)

        # positionate ourself central
        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width() - size.width()) / 2, (screen.height() - size.height()) / 2)
 
        t = QtCore.QTimer(self)
        t.timeout.connect(self.force_foreground)
        t.setInterval(0)
        t.setSingleShot(True)
        t.start()
 

    def clicked_about(self):

        """About button has been clicked."""
        self._about_dialog.show()


    def clicked_browse(self):

        """Browse button has been clicked."""
        if self._open_file_dialog.exec_() == QtGui.QDialog.Accepted:
            self.ui.edtKeyfile.setText(self._open_file_dialog.selectedFiles()[0])


    def clicked_ok(self):
        
        """Ok button has been clicked."""

        # read the content of the keyfile
        keyfile_content = ''
        try:
            keyfile = open(self.ui.edtKeyfile.text(), 'r')
            keyfile_content = keyfile.read()
        except Exception as e:
            sys.stderr.write('failed to read keyfile content:\n' + str(e) + '\n')
            QtGui.QMessageBox.critical(self, 'Failed to read keyfile', str(e))
            return

        # turn into Base64
        keyfile_content_base64 = base64.b64encode(keyfile_content)

        sys.stdout.write('{ ')
        sys.stdout.write('"password": "')
        sys.stdout.write(self.ui.edtPassword.text())
        sys.stdout.write('", ')
        sys.stdout.write('"keyfile": "')
        sys.stdout.write(keyfile_content_base64)
        sys.stdout.write('" ')
        sys.stdout.write('}\n')
        self.accept()


    def force_foreground(self):

        """Force ourselves into foreground"""
        if sys.platform == 'win32' or sys.platform == 'cygwin':
            w = self
            while not w.nativeParentWidget() is None:
                w = w.nativeParentWidget()
            win32gui.BringWindowToTop(int(w.effectiveWinId()))
            win32gui.SetForegroundWindow(int(w.effectiveWinId()))


    def set_user_text(user_text):

        """Set a text to explain which password we need."""
        self.ui.lblText.setText(user_text)


if __name__ == "__main__":
    if sys.platform == 'win32' or sys.platform == 'cygwin':
        win32gui.SystemParametersInfo(win32con.SPI_SETFOREGROUNDLOCKTIMEOUT, 0, win32con.SPIF_SENDWININICHANGE | win32con.SPIF_UPDATEINIFILE)
    a = QtGui.QApplication(sys.argv)
    d = KeyfileDialog()
    d.show()
    sys.exit(a.exec_())     

